{*********************************************************}
{*                 MegaMail Version 2.1                  *}
{*        Copyright(c) Megasistemas Ltda.  1997          *}
{*            Todos los derechos reservados              *}
{*              Author :  Jair Palacio                   *}
{*********************************************************}
{*          http://www.megasistemas.com                  *}
{*   send remarks: juherna@epm.net.co                    *}
{*********************************************************}
{*  Const: Credits                                       *}
{*                                                       *}
{*  Type :  TMegaMail                                    *}
{*                                                       *}
{*********************************************************}
{* Description : this unit contains one component to     *}
{*               send mail to any mail box, inherites    *}
{* directly from TWinControl, and you may attach or      *}
{* include one file in the body.                         *}
{* Using MapiMessage And MapiFileDesc Records            *}
{*********************************************************}


unit MegaEmail;

interface

Uses mapi,Windows, SysUtils, Classes, Graphics, Controls,
     Forms,StdCtrls, ComCtrls;

Const Credits = 'MegaSistemas Ltda(c) By Jair... 1,997. Medelln - Colombia';
type
  TMegaNotifyEvent = procedure(Sender: TObject; Doc_Name: String) of object;
  TMegaErrorNotifyEvent = procedure(Sender: TObject; Error_Number: Cardinal; Error_Message:String) of object;

  TMegaMail = class(TWinControl)
  private
    { Private declarations }
    FAddDate,FAttachment:Boolean;
    FAbout, FDocument: String;
    FOnBeginDoc: TMegaNotifyEvent;             
    FOnEndDoc: TMegaNotifyEvent;
    FOnErrorSend: TMegaErrorNotifyEvent;
    FOnCancelSend: TMegaNotifyEvent;
    FOnFileNotExist: TMegaNotifyEvent;

    procedure SetDocument(Value: String);
    procedure SetAddDate(Value: Boolean);
    function GetAbout: String;
    procedure SetAttachment(Value: Boolean);

    procedure Init_Send;
    procedure Done_Send;
    Procedure FileNotExist;
    Procedure Error_InSend(ENumber:Cardinal;EMessage:String);
    Procedure User_Cancel;
    Procedure Send_Mail(MMail:TMapiMessage);
  protected
    { Protected declarations }
    Procedure SendFile;
    Procedure SendFile_Attach;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure Execute;
  published
    { Published declarations }
    property Source: String read FDocument write SetDocument;
    Property About: String read GetAbout write FAbout;
    Property AddDate: Boolean read FAddDate write SetAddDate;
    Property Attachment: Boolean read FAttachment write SetAttachment;

    property OnBeginDocument: TMegaNotifyEvent read FOnBeginDoc write FOnBeginDoc;
    property OnEndDocument: TMegaNotifyEvent read FOnEndDoc write FOnEndDoc;
    Property OnErrorSend: TMegaErrorNotifyEvent read FOnErrorSend write FOnErrorSend;
    Property OnCancelSend: TMegaNotifyEvent read FOnCancelSend Write FOnCancelSend;
    Property OnFileNotExists: TMegaNotifyEvent read FOnFileNotExist write FOnFileNotExist;
  end;

procedure Register;

implementation

constructor TMegaMail.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAbout:=Credits;
  FDocument:='';
  visible:=False;
  Width:=26;
  Height:=26;
  FAddDate:=False;
  FAttachment:=False;
  Brush.Color:=clBlue;  
  Brush.Style:=bsDiagCross;
end;
destructor TMegaMail.Destroy;
begin
  inherited Destroy;
end;
procedure TMegaMail.Init_Send;
begin
  if Assigned(FOnBeginDoc)then OnBeginDocument(Self,FDocument);
end;
procedure TMegaMail.Done_Send;
begin
  if Assigned(FOnEndDoc)then OnEndDocument(Self,FDocument);
end;
procedure TMegaMail.Error_InSend(ENumber:Cardinal;EMessage:String);
begin
  if Assigned(FOnErrorSend)then OnErrorSend(Self,ENumber,EMessage);
end;
procedure TMegaMail.User_Cancel;
begin
  if Assigned(FOnCancelSend)then OnCancelSend(Self,FDocument);
end;
procedure TMegaMail.FileNotExist;
begin
  if Assigned(FOnFileNotExist)then OnFileNotExists(Self,FDocument);
end;
procedure TMegaMail.SetDocument(Value: String);
begin
  if(Value<>FDocument)Then
  Begin
    If(FileExists(Value))Then FDocument := Value
                         Else FDocument := '';
  End;
end;
Function TMegaMail.GetAbout;
Begin
  GetAbout:=Credits;
End;
procedure TMegaMail.SetAddDate(Value: Boolean);
begin
  if(Value<>FAddDate)Then FAddDate:=Value;
end;
procedure TMegaMail.SetAttachment(Value: Boolean);
begin
  if(Value<>FAttachment)Then FAttachment:=Value;
end;
Procedure TMegaMail.Execute;
Begin
  If(FileExists(FDocument))Then
  Begin
    Hide;
    If(Not(FAttachment))Then SendFile Else SendFile_Attach;
  End Else FileNotExist;
End;
Procedure TMegaMail.SendFile_Attach;
Var MapiFileAtt:PMapiFileDesc;
    MapiMessage:TMapiMessage;
Begin
  New(MapiFileAtt);
  If(MapiFileAtt=Nil)Then Exit;
  With MapiFileAtt^ Do
  Begin
    UlReserved:=0;
    flFlags := Mapi_Ole_Static;
    nPosition:= 0;
    LpszPathName:=PChar(FDocument);
    LpszFileName:=Nil;
    LpFileType:=Nil;
  End;
  with MapiMessage do
  begin
    ulReserved := 0;
    lpszSubject:=Nil;
    lpszNoteText:=Pchar(' '+ExtractFilePath(FDocument));
    lpszMessageType := nil;
    lpszDateReceived := nil;
    lpszConversationID := nil;
    flFlags := 0;
    lpOriginator := nil;
    nRecipCount := 0;
    lpRecips := nil;
    nFileCount := 1;
    lpFiles := MapiFileAtt;
  end;
  Send_Mail(MapiMessage);
End;
Procedure TMegaMail.SendFile;
var MapiMessage: TMapiMessage;
    FRichEdit: TRichEdit;
begin
  FRichEdit:=TRichEdit.Create(GetParentForm(Self));
  Try
    With FRichEdit Do
    Begin
      FRichEdit.Parent:=GetParentForm(Self);
      Visible:=False;
      Randomize;
      Name:='Document'+IntToStr(Random(13));
      Width:=0;
      Top:=0;
      Left:=0;
      Height:=0;
      WordWrap:=False;
      Lines.Clear;
      Lines.LoadFromFile(FDocument);
    End;
    with MapiMessage do
    begin
      ulReserved := 0;
      lpszSubject:=Nil;
      lpszNoteText := PChar(FRichEdit.Lines.Text);
      lpszMessageType := nil;
      lpszDateReceived := nil;
      lpszConversationID := nil;
      flFlags := 0;
      lpOriginator := nil;
      nRecipCount := 0;
      lpRecips := nil;
      nFileCount := 0;
      lpFiles := nil;
    end;
    Send_Mail(MapiMessage);
  Finally
    FRichEdit.free;
  End;
End;
Procedure TMegaMail.Send_Mail(MMail:TMapiMessage);
Var MDisplay,MTemp:String;
    MError:Cardinal;
Begin
  Init_Send;
  Mtemp:=FDocument+' / '+FormatDateTime('yyyy/mm/dd hh:nn',Now);
  If(FAddDate)Then MMail.lpszSubject:= PChar(MTemp)
              Else MMail.lpszSubject:= PChar(FDocument);
  MError:=MapiSendMail(0,0,MMail,MAPI_DIALOG or MAPI_LOGON_UI or MAPI_NEW_SESSION, 0);
  If(MError<>0)Then
  Begin
    FillChar(MDisplay,sizeOf(MDisplay),#0);
    Case MError Of
       MAPI_E_AMBIGUOUS_RECIPIENT:MDisplay:='A recipient matched more than'+
        ' one of the recipient descriptor structures and MAPI_DIALOG was not set.';
       MAPI_E_ATTACHMENT_NOT_FOUND:MDisplay:='The specified attachment was not found.';
       MAPI_E_ATTACHMENT_OPEN_FAILURE:MDisplay:='The specified attachment could not be open.';
       MAPI_E_BAD_RECIPTYPE:MDisplay:='The type of a recipient was not MAPI_TO, MAPI_CC, or MAPI_BCC.';
       MAPI_E_FAILURE:MDisplay:='One or more unspecified errors occurred.';
       MAPI_E_INSUFFICIENT_MEMORY:MDisplay:='There was insufficient memory to proceed.';
       MAPI_E_LOGIN_FAILURE:MDisplay:='There was no default logon, '+
       'and the user failed to log on successfully when the logon dialog box was displayed.';
       MAPI_E_TEXT_TOO_LARGE:MDisplay:='The text in the message was too large to sent.';
       MAPI_E_TOO_MANY_FILES:MDisplay:='There were too many file attachments.';
       MAPI_E_TOO_MANY_RECIPIENTS:MDisplay:='There were too many recipients.';
       MAPI_E_UNKNOWN_RECIPIENT:MDisplay:='A recipient did not appear in the address list.';
       MAPI_E_USER_ABORT:
       Begin
         MDisplay:='The user canceled one of the dialog boxes.';
         User_Cancel;
       End;
    End;
    if(MError<>MAPI_E_USER_ABORT)Then Error_InSend(MError,MDisplay);
  End;
  Done_Send;
End;
procedure Register;
begin
  RegisterComponents('Mega SoftWare', [TMegaMail]);
end;
End.
